
from glouton.modules.observationModuleBase import ObservationModuleBase

class TestModule(ObservationModuleBase):

    def runAfterDownload(self, file_name, full_path, observation):
        print("executed after " +  file_name)

    def runAfterDownloadCompleted(self, full_path):
        print("executed after the download finished")


from datetime import datetime
from glouton.shared import dateHelper


months_equal_0 = [datetime(2019, 11, 30, 0, 0), datetime(2019, 12, 1, 23, 59, 59), datetime(2019, 12, 2, 0, 0), datetime(
    2019, 12, 3, 23, 59, 59), datetime(2019, 12, 4, 0, 0), datetime(2019, 12, 5, 23, 59, 59), datetime(2019, 12, 6, 0, 0), datetime(2019, 12, 7, 0, 0)]

months_equal_1 = [datetime(2019, 11, 1, 1, 21, 54), datetime(2019, 11, 8, 23, 59, 59), datetime(2019, 11, 9, 0, 0), datetime(
    2019, 11, 16, 23, 59, 59), datetime(2019, 11, 17, 0, 0), datetime(2019, 11, 23, 23, 59, 59), datetime(2019, 11, 24, 0, 0), datetime(2019, 12, 1, 1, 21, 54)]

months_equal_2 = [datetime(2017, 10, 19, 0, 51, 54), datetime(2017, 11, 3, 23, 59, 59), datetime(2017, 11, 4, 0, 0), datetime(
    2017, 11, 19, 23, 59, 59), datetime(2017, 11, 20, 0, 0), datetime(2017, 12, 4, 23, 59, 59), datetime(2017, 12, 5, 0, 0), datetime(2017, 12, 20, 0, 51, 54)]

months_equal_3 = [datetime(2017, 9, 19, 0, 51, 54), datetime(2017, 10, 12, 23, 59, 59), datetime(2017, 10, 13, 0, 0), datetime(
    2017, 11, 4, 23, 59, 59), datetime(2017, 11, 5, 0, 0), datetime(2017, 11, 27, 23, 59, 59), datetime(2017, 11, 28, 0, 0), datetime(2017, 12, 20, 0, 51, 54)]

months_equal_4 = [datetime(2017, 8, 19, 0, 51, 54), datetime(2017, 9, 18, 23, 59, 59), datetime(2017, 9, 19, 0, 0), datetime(
    2017, 10, 19, 23, 59, 59), datetime(2017, 10, 20, 0, 0), datetime(2017, 11, 19, 23, 59, 59), datetime(2017, 11, 20, 0, 0), datetime(2017, 12, 20, 0, 51, 54)]

months_equal_5 = [datetime(2017, 8, 19, 0, 51, 54), datetime(2017, 9, 19, 23, 59, 59), datetime(2017, 9, 20, 0, 0), datetime(
    2017, 10, 19, 23, 59, 59), datetime(2017, 10, 20, 0, 0), datetime(2017, 11, 19, 23, 59, 59), datetime(2017, 11, 20, 0, 0), datetime(2017, 12, 20, 0, 51, 54)]

months_equal_31 = [datetime(2015, 5, 1, 1, 21, 54), datetime(2015, 12, 30, 23, 59, 59), datetime(2015, 12, 31, 0, 0), datetime(
    2016, 8, 30, 23, 59, 59), datetime(2016, 8, 31, 0, 0), datetime(2017, 5, 1, 23, 59, 59), datetime(2017, 5, 2, 0, 0), datetime(2017, 12, 31, 1, 21, 54)]


def extract_dates(start_date, end_date):
    div = 4
    dates = []
    for d1, d2 in dateHelper.split_date(start_date, end_date, div):
        dates.append(d1)
        dates.append(d2)

    return dates


def test_days_diff_on_close_months():
    start_date = datetime.strptime(
        '2019-11-29T00:00:00', '%Y-%m-%dT%H:%M:%S')
    end_date = datetime.strptime(
        '2019-12-07T00:00:00', '%Y-%m-%dT%H:%M:%S')
    diff = dateHelper.diff_days(start_date, end_date)

    assert diff == 8

def test_7_days_diff():
    start_date = datetime.strptime(
        '2019-11-29T00:00:00', '%Y-%m-%dT%H:%M:%S')
    end_date = datetime.strptime(
        '2019-12-06T00:00:00', '%Y-%m-%dT%H:%M:%S')
    diff = dateHelper.diff_days(start_date, end_date)

    assert diff == 7

def test_8_days_diff():
    start_date = datetime.strptime(
        '2019-11-29T00:00:00', '%Y-%m-%dT%H:%M:%S')
    end_date = datetime.strptime(
        '2019-12-06T01:00:00', '%Y-%m-%dT%H:%M:%S')
    diff = dateHelper.diff_days(start_date, end_date)
    dates = extract_dates(start_date, end_date)
    print(dates)

    assert diff == 8

def test_days_diff_on_far_months():
    start_date = datetime.strptime(
        '2010-11-29T00:00:00', '%Y-%m-%dT%H:%M:%S')
    end_date = datetime.strptime(
        '2019-12-07T00:00:00', '%Y-%m-%dT%H:%M:%S')
    diff = dateHelper.diff_days(start_date, end_date)

    assert diff == 3295

def test_days_diff_on_same_month():
    start_date = datetime.strptime(
        '2019-12-6T00:00:00', '%Y-%m-%dT%H:%M:%S')
    end_date = datetime.strptime(
        '2019-12-07T00:00:00', '%Y-%m-%dT%H:%M:%S')
    diff = dateHelper.diff_days(start_date, end_date)

    assert diff == 1

def test_days_diff_no_days():
    start_date = datetime.strptime(
        '2019-12-07T00:00:00', '%Y-%m-%dT%H:%M:%S')
    end_date = datetime.strptime(
        '2019-12-07T00:00:00', '%Y-%m-%dT%H:%M:%S')
    diff = dateHelper.diff_days(start_date, end_date)

    assert diff == 0


def test_split_0_months_to_4_ranges():
    start_date = datetime.strptime(
        '2019-11-30T00:00:00', '%Y-%m-%dT%H:%M:%S')
    end_date = datetime.strptime(
        '2019-12-07T00:00:00', '%Y-%m-%dT%H:%M:%S')
    dates = extract_dates(start_date, end_date)

    assert dates == months_equal_0
    assert dates[0] == start_date
    assert dates[len(dates)-1] == end_date
    assert len(dates) == 8


def test_split_1_months_to_4_ranges():
    start_date = datetime.strptime(
        '2019-11-01T01:21:54', '%Y-%m-%dT%H:%M:%S')
    end_date = datetime.strptime(
        '2019-12-01T01:21:54', '%Y-%m-%dT%H:%M:%S')
    dates = extract_dates(start_date, end_date)

    assert dates == months_equal_1
    assert dates[0] == start_date
    assert dates[len(dates)-1] == end_date
    assert len(dates) == 8


def test_split_3_months_to_4_ranges():
    start_date = datetime.strptime(
        '2017-09-19T00:51:54', '%Y-%m-%dT%H:%M:%S')
    end_date = datetime.strptime(
        '2017-12-20T00:51:54', '%Y-%m-%dT%H:%M:%S')
    dates = extract_dates(start_date, end_date)

    assert dates == months_equal_3
    assert dates[0] == start_date
    assert dates[len(dates)-1] == end_date
    assert len(dates) == 8


def test_split_2_months_to_4_ranges():
    start_date = datetime.strptime(
        '2017-10-19T00:51:54', '%Y-%m-%dT%H:%M:%S')
    end_date = datetime.strptime(
        '2017-12-20T00:51:54', '%Y-%m-%dT%H:%M:%S')
    dates = extract_dates(start_date, end_date)

    assert dates == months_equal_2
    assert dates[0] == start_date
    assert dates[len(dates)-1] == end_date
    assert len(dates) == 8


def test_split_4_months_to_4_ranges():
    start_date = datetime.strptime(
        '2017-08-19T00:51:54', '%Y-%m-%dT%H:%M:%S')
    end_date = datetime.strptime(
        '2017-12-20T00:51:54', '%Y-%m-%dT%H:%M:%S')
    dates = extract_dates(start_date, end_date)

    assert dates == months_equal_4
    assert dates[0] == start_date
    assert dates[len(dates)-1] == end_date
    assert len(dates) == 8


def test_split_31_months_to_4_ranges():
    start_date = datetime.strptime(
        '2015-05-01T01:21:54', '%Y-%m-%dT%H:%M:%S')
    end_date = datetime.strptime(
        '2017-12-31T01:21:54', '%Y-%m-%dT%H:%M:%S')
    dates = extract_dates(start_date, end_date)

    assert dates == months_equal_31
    assert dates[0] == start_date
    assert dates[len(dates)-1] == end_date
    assert len(dates) == 8



class EndModuleWorker:
    def __init__(self, queue, download_end_status):
        self.__commands = queue
        self.__download_end_status = download_end_status

    def execute(self):
        self.__download_end_status.wait()
        while self.__commands.empty() == False:
            try:
                command = self.__commands.get()
                command.process()
                self.__commands.task_done()
            except:
                pass

        self.__download_end_status.clear()



from glouton.shared.logger import logger


class DownloadWorker:
    def __init__(self, queue, download_status, is_download_finished):
        self._commands = queue
        self.__download_status = download_status
        self.__is_download_finished = is_download_finished

    def execute(self):
        self.__download_status.set()
        try:
            while self._commands.empty() == False:
                command = self._commands.get()
                command.download()
                self._commands.task_done()

        except Exception as ex:
            logger.Error(ex)

        self.__download_status.clear()
        if self.__is_download_finished is not None:
            self.__is_download_finished.set()



from glouton.modules.telemetryModuleBase import TelemetryModuleBase
from glouton.modules.painaniDecoder import PainaniDecoder
import binascii
import os


class PainaniImageDecoder(TelemetryModuleBase):
    def __init__(self, wdir):
        TelemetryModuleBase.__init__(self, wdir)
        self.__decoder = PainaniDecoder()
        self.__frame_counter = 0

    def runAfterDownload(self, frame, full_path, telemetry):
        timestamp = telemetry['timestamp']
        self.__frame_counter += 1
        file = os.path.join(full_path, timestamp.replace(
            ':', '-') + "_" + str(self.__frame_counter) + ".frame")
        with open(file, 'wb') as f:
            f.write(binascii.unhexlify(frame))

    def runAfterDownloadCompleted(self, full_path):
        self.__decoder.DecodeImages(full_path, "*.frame")



from glouton.modules.observationModuleBase import ObservationModuleBase
from glouton.modules.painaniDecoder import PainaniDecoder


class NtPainaniImageDecoder(ObservationModuleBase):
    def __init__(self, wdir):
        ObservationModuleBase.__init__(self, wdir)
        self.decoder = PainaniDecoder()

    def runAfterDownloadCompleted(self, full_path):
        self.decoder.DecodeImages(full_path, "data_*")



import os
import mock
import pytest
import requests
from glouton.infrastructure.satnogNetworkClient import SatnogNetworkClient
from glouton.infrastructure.satnogDbClient import SatnogDbClient


@mock.patch.dict(os.environ, {"SATNOGS_NETWORK_API_URL": "network_url_from_env"})
def test_network_api_url_via_env_variable():
    with pytest.raises(requests.exceptions.MissingSchema) as ex:
        client = SatnogNetworkClient()
        client.get_from_base("/test")
    assert "network_url_from_env/test" in str(ex.value)


@mock.patch.dict(os.environ, {"SATNOGS_DB_API_URL": "db_url_from_env"})
def test_db_api_url_via_env_variable():
    with pytest.raises(requests.exceptions.MissingSchema) as ex:
        client = SatnogDbClient()
        client.get_from_base("/test")
    assert "db_url_from_env/test" in str(ex.value)



from glouton.shared.logger import logger


class PageScanWorker:
    def __init__(self, client, cmd, repos, path, url_params, job_number):
        self.PATH = path
        self.__client = client
        self.__cmd = cmd
        self.__repos = repos
        self.__job_number = str(job_number)
        self.__url_params = url_params

    def scan(self):
        params = self.__url_params
        page = 1
        job_string = 'job ' + self.__job_number
        while True:
            r = self.__client.get_from_base(
                self.PATH, params)
            if r.status_code != 200:
                break

            logger.Info(job_string +
                        ' scanning page...' + params['page'])
            self.__read_page(r.json(), self.__cmd.start_date,
                             self.__cmd.end_date)
            page += 1
            params['page'] = str(page)

        logger.Info(job_string + ' terminated')

    def __read_page(self, elements, start_date, end_date):
        for element in elements:
            for repo in self.__repos:
                repo.register_download_command(
                    element, start_date, end_date)



from queue import Queue
from glouton.shared import dateHelper
from glouton.workers.pageScanWorker import PageScanWorker
from glouton.infrastructure.satnogDbClient import SatnogDbClient
from glouton.shared import threadHelper
from glouton.shared.logger import logger


class TelemetryRepo:
    def __init__(self, cmd, repos):
        self.TELEMETRY_URL = 'telemetry/'
        self.__repos = repos
        self.__cmd = cmd
        self.__threads = []

    def extract(self):
        client = SatnogDbClient()
        diff_days = dateHelper.diff_days(
            self.__cmd.start_date, self.__cmd.end_date)
        if diff_days < 8:
            # no thread needed
            url_params = self.__url_param_builder(
                self.__cmd.start_date, self.__cmd.end_date)
            pageScanner = PageScanWorker(
                client, self.__cmd, self.__repos, self.TELEMETRY_URL, url_params, 1)
            pageScanner.scan()
        else:
            threads = []
            job = 1
            for from_datetime, to_datetime in dateHelper.split_date(self.__cmd.start_date, self.__cmd.end_date, 4):
                print(str(from_datetime) + " " + str(to_datetime))
                url_params = self.__url_param_builder(
                    from_datetime, to_datetime)
                pageScanner = PageScanWorker(
                    client, self.__cmd, self.__repos, self.TELEMETRY_URL, url_params, job)
                t = threadHelper.create_thread(pageScanner.scan)
                threads.append(t)
                job += 1

            threadHelper.wait(threads)
        print('\ndownloading started (Ctrl + C to stop)...\t~(  ^o^)~')
        self.__register_end_command()
        self.__create_workers_and_wait()

    def __register_end_command(self):
        for repo in self.__repos:
            repo.register_end_command(
                self.__cmd.start_date, self.__cmd.end_date)

    def __create_workers_and_wait(self):
        for repo in self.__repos:
            self.__threads.extend(repo.create_worker())
        threadHelper.wait(self.__threads)

    def __url_param_builder(self, start_date, end_date):
        return {'satellite': self.__cmd.norad_id,
                'start': start_date.isoformat(),
                'end': end_date.isoformat(),
                'observer': self.__cmd.observer,
                'transmitter': self.__cmd.transmitter,
                'app_source': self.__cmd.app_source,
                'page': '1',
                'format': 'json'}



from glouton.modules.telemetryModuleBase import TelemetryModuleBase
from glouton.shared.logger import logger
import json
import os


class FlagNoImageNexusFrame(TelemetryModuleBase):
    def __init__(self, wdir):
        TelemetryModuleBase.__init__(self, wdir)
        self.types_found = []

    def runAfterDownload(self, frame, full_path, telemetry):
        timestamp = telemetry['timestamp']
        type = frame[32:34]
        if type != "C1":
            if type not in self.types_found:
                self.types_found.append(type)
            print(self.types_found)
            print("found type : " + type + " of telemetry : " + timestamp)
            file = full_path + '/' + type + "_" + timestamp.replace(':', '-') + '.frame'
            with open(file, 'w') as f:
                f.write(frame)



import os
import sys
import json


def read():
    config = None
    default_config = {
        "DEFAULT": {
            "NETWORK_API_URL": "https://network.satnogs.org/api/",
            "DB_API_URL": "https://db.satnogs.org/api/",
            "DB_API_KEY": "",
            "HTTPS_PROXY": "",
            "HTTP_PROXY": ""
        },
        "MODULES": {
            "FOR_EACH": {
                "ARCHIVE": [],
                "WATERFALL": [],
                "DEMODDATA": [],
                "FRAME": [],
                "FOR_ALL_OBSERVATION": []
            },
            "END": {
                "ARCHIVE": [],
                "WATERFALL": [],
                "DEMODDATA": [],
                "FRAME": [],
                "FOR_ALL_OBSERVATION": []
            }
        },
        "LOGFILE": "glouton.log"
    }
    try:
        if os.path.exists(os.path.dirname(sys.argv[0]) + "/../glouton/config.json"):
            with open(os.path.dirname(sys.argv[0]) + "/../glouton/config.json", 'r') as f:
                config = json.load(f)
        else:
            config = default_config

    except Exception as eee:
        print("error: ", eee)
        config = default_config
    return config



from glouton.services.module.moduleService import ModuleService
from glouton.repositories.demoddata.demoddataRepo import DemoddataRepo
from glouton.repositories.archive.archiveRepo import ArchiveRepo
from glouton.repositories.waterfall.waterfallRepo import WaterfallRepo
from glouton.repositories.observation.observationsRepo import ObservationRepo


class ObservationsService:
    def __init__(self, cmd):
        self.__cmd = cmd
        self.__module_service = ModuleService(self.__cmd.working_dir)
        repos = self.filter_repositories()
        self.__observations_repo = ObservationRepo(self.__cmd, repos)

    def filter_repositories(self):
        downloadable_data_repos = []
        all = not self.__cmd.archives and not self.__cmd.waterfalls and not self.__cmd.demoddata
        if all or self.__cmd.archives:
            downloadable_data_repos.append(ArchiveRepo(
                self.__cmd.working_dir, self.__module_service.loadArchiveModules(self.__cmd.archive_modules, 'FOR_EACH'), self.__module_service.loadArchiveModules(self.__cmd.archive_end_modules, 'END')))
        if all == True or self.__cmd.waterfalls:
            downloadable_data_repos.append(WaterfallRepo(
                self.__cmd.working_dir, self.__module_service.loadWaterfallModules(self.__cmd.waterfall_modules, 'FOR_EACH'), self.__module_service.loadWaterfallModules(self.__cmd.waterfall_end_modules, 'END')))
        if all == True or self.__cmd.demoddata:
            downloadable_data_repos.append(DemoddataRepo(
                self.__cmd.working_dir, self.__module_service.loadDemoddataModules(self.__cmd.demoddata_modules, 'FOR_EACH'), self.__module_service.loadDemoddataModules(self.__cmd.demoddata_end_modules, 'END')))

        return downloadable_data_repos

    def extract(self):
        self.__observations_repo.extract()



from glouton.services.observation.observationsService import ObservationsService
from glouton.repositories.archive.archiveRepo import ArchiveRepo
from glouton.repositories.waterfall.waterfallRepo import WaterfallRepo
from glouton.repositories.demoddata.demoddataRepo import DemoddataRepo
from glouton.domain.parameters.programCmd import ProgramCmd


def test_loading_all_repository_types():
    params = ProgramCmd(None, None, None, None, None,
                        None,
                        True,
                        True,
                        True,
                        None,
                        None,
                        None,
                        None,
                        None,
                        None,
                        None,
                        None,
                        None,
                        None,
                        None,
                        None,
                        None,
                        None,
                        None,
                        )

    service = ObservationsService(params)
    repos = service.filter_repositories()
    assert len(repos) == 3
    assert isinstance(repos[0], ArchiveRepo)
    assert isinstance(repos[1], WaterfallRepo)
    assert isinstance(repos[2], DemoddataRepo)


def test_loading_only_archive_repository_types():
    params = ProgramCmd(None, None, None, None, None,
                        None,
                        True,
                        False,
                        False,
                        None,
                        None,
                        None,
                        None,
                        None,
                        None,
                        None,
                        None,
                        None,
                        None,
                        None,
                        None,
                        None,
                        None,
                        None,
                        )

    service = ObservationsService(params)
    repos = service.filter_repositories()
    assert len(repos) == 1
    assert isinstance(repos[0], ArchiveRepo)


def test_loading_only_waterfall_repository_types():
    params = ProgramCmd(None, None, None, None, None,
                        None,
                        False,
                        True,
                        False,
                        None,
                        None,
                        None,
                        None,
                        None,
                        None,
                        None,
                        None,
                        None,
                        None,
                        None,
                        None,
                        None,
                        None,
                        None,
                        )

    service = ObservationsService(params)
    repos = service.filter_repositories()
    assert len(repos) == 1
    assert isinstance(repos[0], WaterfallRepo)


def test_loading_only_demoddata_repository_types():
    params = ProgramCmd(None, None, None, None, None,
                        None,
                        False,
                        False,
                        True,
                        None,
                        None,
                        None,
                        None,
                        None,
                        None,
                        None,
                        None,
                        None,
                        None,
                        None,
                        None,
                        None,
                        None,
                        None,
                        )

    service = ObservationsService(params)
    repos = service.filter_repositories()
    assert len(repos) == 1
    assert isinstance(repos[0], DemoddataRepo)


def test_loading_only_archive_demoddata_repository_types():
    params = ProgramCmd(None, None, None, None, None,
                        None,
                        True,
                        False,
                        True,
                        None,
                        None,
                        None,
                        None,
                        None,
                        None,
                        None,
                        None,
                        None,
                        None,
                        None,
                        None,
                        None,
                        None,
                        None,
                        )

    service = ObservationsService(params)
    repos = service.filter_repositories()
    assert len(repos) == 2
    assert isinstance(repos[0], ArchiveRepo)
    assert isinstance(repos[1], DemoddataRepo)


def test_loading_only_waterfall_demoddata_repository_types():
    params = ProgramCmd(None, None, None, None, None,
                        None,
                        False,
                        True,
                        True,
                        None,
                        None,
                        None,
                        None,
                        None,
                        None,
                        None,
                        None,
                        None,
                        None,
                        None,
                        None,
                        None,
                        None,
                        None,
                        )

    service = ObservationsService(params)
    repos = service.filter_repositories()
    assert len(repos) == 2
    assert isinstance(repos[0], WaterfallRepo)
    assert isinstance(repos[1], DemoddataRepo)


def test_default_loading_repository():
    params = ProgramCmd(None, None, None, None, None,
                        None,
                        False,
                        False,
                        False,
                        None,
                        None,
                        None,
                        None,
                        None,
                        None,
                        None,
                        None,
                        None,
                        None,
                        None,
                        None,
                        None,
                        None,
                        None,
                        )

    service = ObservationsService(params)
    repos = service.filter_repositories()
    assert len(repos) == 3
    assert isinstance(repos[0], ArchiveRepo)
    assert isinstance(repos[1], WaterfallRepo)
    assert isinstance(repos[2], DemoddataRepo)



from glouton.commands.module.moduleCommand import ModuleCommand


class EndModuleCommand(ModuleCommand):
    def __init__(self, params):
        ModuleCommand.__init__(self, params)

    def process(self):
        for module in self.params.modules:
            module.runAfterDownloadCompleted(self.params.full_path)



class ProgramCmd:
    def __init__(self,
                 norad_id,
                 ground_station_id,
                 start_date,
                 end_date,
                 observation_status,
                 working_dir,
                 archives,
                 waterfalls,
                 demoddata,
                 archive_modules,
                 archive_end_modules,
                 demoddata_modules,
                 demoddata_end_modules,
                 waterfall_modules,
                 waterfall_end_modules,
                 user,
                 transmitter_uuid,
                 transmitter_mode,
                 transmitter_type,
                 frame_modules,
                 frame_end_modules,
                 observer,
                 app_source,
                 transmitter):
        self.norad_id = norad_id
        self.ground_station_id = ground_station_id
        self.start_date = start_date
        self.end_date = end_date
        self.working_dir = working_dir
        self.archives = archives
        self.waterfalls = waterfalls
        self.demoddata = demoddata
        self.archive_modules = archive_modules
        self.archive_end_modules = archive_end_modules
        self.demoddata_modules = demoddata_modules
        self.demoddata_end_modules = demoddata_end_modules
        self.waterfall_modules = waterfall_modules
        self.waterfall_end_modules = waterfall_end_modules
        self.observation_status = observation_status
        self.user = user
        self.transmitter_uuid = transmitter_uuid
        self.transmitter_mode = transmitter_mode
        self.transmitter_type = transmitter_type
        self.frame_modules = frame_modules
        self.frame_end_modules = frame_end_modules
        self.observer = observer
        self.app_source = app_source
        self.transmitter = transmitter



from queue import Queue
from threading import Thread
from glouton.commands.download.downloadCommandParams import DownloadCommandParams
from glouton.commands.download.archiveDownloadCommand import ArchiveDownloadCommand
from glouton.commands.module.endModuleCommand import EndModuleCommand
from glouton.commands.module.endModuleCommandParams import EndModuleCommandParams
from glouton.workers.downloadWorker import DownloadWorker
from glouton.workers.moduleWorker import ModuleWorker
from glouton.workers.endModuleWorker import EndModuleWorker
from glouton.domain.interfaces.downloadable import Downloadable
from glouton.shared import threadHelper
from threading import Event
import os


class ArchiveRepo(Downloadable):
    def __init__(self, working_dir, modules, end_modules):
        self.__working_dir = working_dir
        self.__archive_commands = Queue()
        self.__archive_modules_commands = Queue()
        self.__archive_end_modules_commands = Queue()
        self.__modules = modules
        self.__end_modules = end_modules
        self.__download_status = Event()
        self.__is_download_finished = Event()

    def register_download_command(self, observation, start_date, end_date):
        cmd_parameters = DownloadCommandParams(
            self.__working_dir, self.__create_dir_name('archive', start_date, end_date), self.__modules)
        waterfallDownloadCommand = ArchiveDownloadCommand(
            cmd_parameters, observation, self.__archive_modules_commands)
        self.__archive_commands.put(waterfallDownloadCommand)

    def register_end_command(self, start_date, end_date):
        if self.__end_modules is not None:
            dir_name = self.__create_dir_name('archive', start_date, end_date)
            module_parameters = EndModuleCommandParams(full_path=os.path.join(
                self.__working_dir, dir_name), modules=self.__end_modules)
            archive_end_module_command = EndModuleCommand(
                module_parameters)
            self.__archive_end_modules_commands.put(archive_end_module_command)

    def create_worker(self):
        threads = []
        downloadWorker = DownloadWorker(
            self.__archive_commands, self.__download_status, self.__is_download_finished if self.__modules is None else None)
        threads.append(threadHelper.create_thread(downloadWorker.execute))
        if self.__modules is not None:
            moduleWorker = ModuleWorker(
                self.__archive_modules_commands, self.__download_status, self.__is_download_finished)
            threads.append(threadHelper.create_thread(moduleWorker.execute))

        if self.__end_modules is not None:
            endWorker = EndModuleWorker(
                self.__archive_end_modules_commands, self.__is_download_finished)
            threads.append(threadHelper.create_thread(endWorker.execute()))

        return threads

    def __create_dir_name(self, target, start_date, end_date):
        return target + '__' + start_date.strftime('%m-%d-%YT%H-%M-%S') + '__' + end_date.strftime('%m-%d-%YT%H-%M-%S')



from threading import Thread


def create_thread(target):
    thread = Thread(target=target)
    thread.daemon = True
    thread.start()
    return thread


def is_one_thread_alive(threads):
    for thread in threads:
        if thread.is_alive():
            return True

    return False


def wait(threads):
    while is_one_thread_alive(threads):
        for t in threads:
            # let's control to main thread every seconds (in order to be able to capture Ctrl + C if needed)
            t.join(1)



import importlib
from glouton.shared.logger import logger
from glouton.shared import config


class ModuleService:
    def __init__(self, working_dir):
        self.__working_dir = working_dir
        self.__config = config.read()

    def loadDemoddataModules(self, demoddata_modules, when):
        logger.Info('(' + when + ') Demoddata module(s) loading :')
        demoddata_modules = self.__getModulesFromConfig(
            demoddata_modules, when, 'DEMODDATA')
        demoddata_modules = self.__getModulesFromConfig(
            demoddata_modules, when, 'FOR_ALL_OBSERVATION')

        return self.__loadModule(demoddata_modules)

    def loadArchiveModules(self, archive_modules, when):
        logger.Info('(' + when + ') Archive module(s) loading :')
        archive_modules = self.__getModulesFromConfig(
            archive_modules, when, 'ARCHIVE')
        archive_modules = self.__getModulesFromConfig(
            archive_modules, when, 'FOR_ALL_OBSERVATION')

        return self.__loadModule(archive_modules)

    def loadWaterfallModules(self, waterfall_modules, when):
        logger.Info('(' + when + ') Waterfall module(s) loading :')
        waterfall_modules = self.__getModulesFromConfig(
            waterfall_modules, when, 'WATERFALL')
        waterfall_modules = self.__getModulesFromConfig(
            waterfall_modules, when, 'FOR_ALL_OBSERVATION')

        return self.__loadModule(waterfall_modules)

    def loadFrameModules(self, frame_modules, when):
        logger.Info('(' + when + ') Frame module(s) loading :')
        frame_modules = self.__getModulesFromConfig(
            frame_modules, when, 'FRAME')

        return self.__loadModule(frame_modules)

    def __loadModule(self, modules):
        if modules is None:
            logger.Info('No module list found')
            return None

        loaded_modules = []
        for name in modules:
            loaded_module = importlib.import_module(
                'glouton.modules.' + name.lower())
            module = getattr(loaded_module, name)
            loaded_modules.append(module(self.__working_dir))
            logger.Info('module : ' + name + ' loaded')
        return loaded_modules

    def __getModulesFromConfig(self, modules, when, config_array_name):
        try:
            modules_from_config = self.__config['MODULES'][when][config_array_name]
        except:
            logger.Warning('config.json : modules bad format')
            modules_from_config = []

        if len(modules_from_config) == 0 and modules is None:
            return None

        if len(modules_from_config) == 0 and modules is not None:
            return modules

        if modules is None:
            modules = []

        for module in modules_from_config:
            if module in modules:
                logger.Warning('warning : ' + module + ' already referenced.')

            modules.append(module)

        return modules



import mock
from glouton.shared.config import read

default_config = {
        "DEFAULT": {
            "NETWORK_API_URL": "https://network.satnogs.org/api/",
            "DB_API_URL": "https://db.satnogs.org/api/",
            "DB_API_KEY": "",
            "HTTPS_PROXY": "",
            "HTTP_PROXY": ""
        },
        "MODULES": {
            "FOR_EACH": {
                "ARCHIVE": [],
                "WATERFALL": [],
                "DEMODDATA": [],
                "FRAME": [],
                "FOR_ALL_OBSERVATION": []
            },
            "END": {
                "ARCHIVE": [],
                "WATERFALL": [],
                "DEMODDATA": [],
                "FRAME": [],
                "FOR_ALL_OBSERVATION": []
            }
        },
        "LOGFILE": "glouton.log"
    }

def exist_return_false():
    return False

def test_default_configuration():
    patcher = mock.patch('os.path.exists')
    exists_mock = patcher.start()
    exists_mock.side_effect = exist_return_false
    config_obj = read()

    assert config_obj == default_config


import json

from glouton.modules.observationModuleBase import ObservationModuleBase


class ObsMetadataToJsonFile(ObservationModuleBase):

    def runAfterDownload(self, file_name, full_path, observation):
        json_file = full_path + '/' + file_name + '.json'
        with open(json_file, 'w') as f:
            f.write(json.dumps(observation, indent=4))



import datetime
import calendar


def diff_days(d1, d2):
    return (d1-d2).days*-1


def date_delta(start_date, end_date, no_of_ranges):
    start_epoch = calendar.timegm(start_date.timetuple())
    end_epoch = calendar.timegm(end_date.timetuple())
    date_diff = end_epoch - start_epoch
    step = date_diff / no_of_ranges
    return datetime.timedelta(seconds=step)


def date_span(start_date, end_date, delta=datetime.timedelta(days=1)):
    currentdate = start_date
    first = True
    while currentdate + delta < end_date:
        todate = (currentdate + delta).replace(hour=23, minute=59, second=59)
        if first:
            yield currentdate, todate
            first = False
        else:
            current = currentdate + datetime.timedelta(days=1)
            yield current.replace(hour=0, minute=0, second=0), todate

        currentdate += delta

    todate = todate.replace(hour=00, minute=00, second=00) + \
        datetime.timedelta(days=1)
    yield todate, end_date

def split_date(start_date, end_date, div):
    delta = date_delta(start_date, end_date, div)
    print(delta)
    return date_span(start_date, end_date, delta)



import os
import glob
import binascii


class PainaniDecoder:
    def __init__(self):
        pass

    def DecodeImages(self, full_path, file_pattern):
        files = glob.glob(os.path.join(full_path, file_pattern))
        mode = 'rb'
        images = {}
        chunks = []
        for file in files:
            with open(file, mode) as f:
                payload = bytearray(f.read())
                magic = bytearray([0x03, 0xf0, 0xcc, 0xcc])
                magic_id = payload[14:18]
                offset = 37
                if magic_id == magic:
                    image_id = int.from_bytes(payload[25:28], 'big')
                    if image_id not in images.keys():
                        images[image_id] = {}
                    chunk_id = int.from_bytes(payload[28:30], 'big')
                    chunk_size = payload[36]
                    chunk = payload[offset:offset + chunk_size]
                    if len(chunk) == chunk_size:
                        images[image_id][chunk_id] = chunk

        for image_key, chunk in sorted(images.items()):
            image_name = os.path.join(
                full_path, "image_" + str(image_key) + ".j2c")
            print("writing image:", image_name)
            img_sz = 0
            chunks = 0
            f = open(image_name, 'w+b')
            for ch_key, ch_value in sorted(chunk.items()):
                chunks += 1
                binary_data = []
                binary_data = bytearray(ch_value)
                f.write(binary_data)
                img_sz += len(binary_data)
            print("Image", image_key, "size:",
                  chunks, "chunks,", img_sz, "bytes")
            f.close()
